package com.franckbarbier.My_stack;

import com.pauware.pauware_engine.Core.*;
import com.pauware.pauware_engine.Exceptions.*;

public class My_stack implements Manageable {

    /**
     * business data
     */
    java.util.Stack<Object> _stack;
    /**
     * UML state machine
     */
    AbstractState _Empty;
    AbstractState _Only_one;
    AbstractState _More_than_one;
    AbstractState _Not_empty;
    AbstractStateMachine _My_stack_state_machine;

    /**
     * UML creation
     */
    private void init_structure() throws State_exception {
        _stack = new java.util.Stack<>();
    }

    private void init_behavior() throws State_exception {
        _Empty = new State("Empty");
        _Empty.inputState();
        _Empty.stateInvariant(this, "size_is_equal_to_0");
        _Only_one = new State("Only one");
        _Only_one.stateInvariant(this, "size_is_equal_to_1");
        _More_than_one = new State("More than one");
        _More_than_one.stateInvariant(this, "size_is_greater_than_1");
        _Not_empty = (_Only_one.xor(_More_than_one)).name("Not empty");
        _Not_empty.allowedEvent("peek", this, "_peek");

        // PauWare2Web:
        _My_stack_state_machine = new StateMachine(_Empty.xor(_Not_empty), "My stack", AbstractStateMachine.Don_t_show_on_system_out, new com.pauware.pauware2web.PauWare2Web_client());
        _My_stack_state_machine.allowedEvent("empty_", this, "_empty");

        // For PauWare2Web only, fake set of anonymous values as last argument:
        _My_stack_state_machine.allowedEvent("search", this, "_search", new Object[]{new Object()}); // this allowed event is overridden in the 'public void search(Object o) throws State_exception' method below
        // Note: 'search' event might be assigned to 'Not empty' state instead
    }

    public void start() throws State_exception {
        // For test only, overriden by '_My_stack_state_machine.allowedEvent("empty_",this,"_empty");': 
        // _My_stack_state_machine.fires("empty_", _My_stack_state_machine, _My_stack_state_machine, true, this, "_empty");

        _My_stack_state_machine.fires("pop", _Only_one, _Empty, true, this, "_pop");
        _My_stack_state_machine.fires("pop", _More_than_one, _Only_one, this, "size_is_equal_to_2", this, "_pop");
        _My_stack_state_machine.fires("pop", _More_than_one, _More_than_one, this, "size_is_not_equal_to_2", this, "_pop");

        // For PauWare PauWare2Web only, fake set of anonymous values as last argument:
        _My_stack_state_machine.fires("push", _Empty, _Only_one, true, this, "_push", new Object[]{new Object()}); // this transition is overridden in the 'public void push(Object item) throws State_exception' method below
        _My_stack_state_machine.fires("push", _Only_one, _More_than_one, true, this, "_push", new Object[]{new Object()}); // this transition is overridden in the 'public void push(Object item) throws State_exception' method below
        _My_stack_state_machine.fires("push", _More_than_one, _More_than_one, true, this, "_push", new Object[]{new Object()}); // this transition is overridden in the 'public void push(Object item) throws State_exception' method below

        _My_stack_state_machine.start();
    }

    public void stop() throws State_exception {
        _My_stack_state_machine.stop();
    }

    public My_stack() throws State_exception {
        init_structure();
        init_behavior();
    }

    /**
     * UML events
     */
    public void empty_() throws State_exception {
        _My_stack_state_machine.run_to_completion("empty_", AbstractStateMachine.Compute_invariants);
    }

    public void peek() throws State_exception {
        _My_stack_state_machine.run_to_completion("peek", AbstractStateMachine.Compute_invariants);
    }

    public void pop() throws State_exception {
        _My_stack_state_machine.run_to_completion("pop", AbstractStateMachine.Compute_invariants);
    }

    public void push(Object item) throws State_exception {
        _My_stack_state_machine.fires("push", _Empty, _Only_one, true, this, "_push", new Object[]{item});
        _My_stack_state_machine.fires("push", _Only_one, _More_than_one, true, this, "_push", new Object[]{item});
        _My_stack_state_machine.fires("push", _More_than_one, _More_than_one, true, this, "_push", new Object[]{item});
        _My_stack_state_machine.run_to_completion("push", AbstractStateMachine.Compute_invariants);
    }

    public void search(Object o) throws State_exception {
        _My_stack_state_machine.allowedEvent("search", this, "_search", new Object[]{o});
// For test only, conflict with ' _My_stack_state_machine.allowedEvent("search",this,"_search",new Object[]{o});', overriding will occur:
//        _My_stack_state_machine.fires("search", _My_stack_state_machine, _My_stack_state_machine, true, this, "_search", new Object[]{o});
        _My_stack_state_machine.run_to_completion("search", AbstractStateMachine.Compute_invariants);
    }

    /**
     * UML actions (called by reflection; they must be 'public')
     */
    public boolean _empty() {
        return _stack.empty();
    }

    public Object _peek() {
        return _stack.peek();
    }

    public Object _pop() {
//        For test only:
//        Object o = null;
//        o.notify(); // deliberated bug!
        return _stack.pop();
    }

    public Object _push(Object item) {
        return _stack.push(item);
    }

    public int _search(Object o) {
        return _stack.search(o);
    }

    /**
     * UML guards
     */
    public boolean size_is_equal_to_2() {
        return _stack.size() == 2;
    }

    public boolean size_is_not_equal_to_2() {
        return _stack.size() != 2;
    }

    /**
     * UML invariants
     */
    public boolean size_is_equal_to_0() {
        return _stack.size() == 0;
    }

    public boolean size_is_equal_to_1() {
        return _stack.size() == 1;
    }

    public boolean size_is_greater_than_1() {
        return _stack.size() > 1;
    }

    /**
     * Management
     */
    public String async_current_state() {
        return _My_stack_state_machine.async_current_state();
    }

    public String current_state() {
        return _My_stack_state_machine.current_state();
    }

    public boolean in_state(String name) {
        return _My_stack_state_machine.in_state(name);
    }

    public void to_state(String name) throws State_exception {
        _My_stack_state_machine.to_state(name);
    }

    public String name() {
        return _My_stack_state_machine.name();
    }

    public String verbose() {
        return _My_stack_state_machine.verbose();
    }

    public void reset() throws State_exception {
        try {
            while (true) {
                _stack.pop();
            }
        } catch (java.util.EmptyStackException ese) {
            to_state("Empty");
        }
    }

    /**
     * Adaptation
     */
    public void adapt() throws State_exception {
        _My_stack_state_machine.unfires("pop", _Only_one, _Empty, true, this, "_pop", null, AbstractStateMachine.No_reentrance);
    }

    public static void main(String args[]) {
        System.out.println("Working directory: " + System.getProperty("user.dir"));
        try {
            My_stack ms = new My_stack();
            ms.start();
            Object x = new Object();
            Object y = new Object();
//            ps.push(x);
//            ps.push(y);
//            ps.pop();
//            ps.adapt();
//            ps.pop(); // no effect because of 'adapt'
//            ps.reset();
            for (int i = 0; i < 2; i++) {
                ms.empty_();
                ms.push(x);
                ms.search(x);
                ms.push(y);
                ms.push(y);
                ms.pop();
                ms.pop();
                ms.pop();
            }
            ms.stop();
        } catch (Throwable t) {
            t.printStackTrace();
        }
    }
}
